/*****
This program determines the chirality of CA atom in the amino acids.
The chirality is determined by calculating the normal between 
N-CA-C and C-CA-CB planes.
----------------------------------------------------------------------
*****/


#include<stdio.h>
#include<math.h>
#include<string.h>
#include<stdlib.h>


struct Chirality
{
	int aa_Num;
	char Aminoacid[5];
	float theta;
};

int file(int num_atoms);
struct Chirality *scan(FILE *, int, int, char **, int); 
struct Chirality *dihedral(char **atoms, int num);

int main(int argc[], char* argv[])
{
	int j=0, array_Index=0, num_atoms=0;
	char ** atoms; 
	static struct Chirality *struct_ptr;   
       	FILE *fp_N, *fp_C, *fp_CA, *fp_CB, *fp_Result;
   
	sscanf(argv[1], "%d", &num_atoms);
	
	num_atoms=file(num_atoms);			/*extracting the information of chiral amino acids only*/
      	
   	system("bash coord_extract.sh");		/*extracting the induvidual atom's(N, CA, CB, C) coordinates in separate files*/	
		
	atoms = malloc(num_atoms * sizeof(char*));
	if(atoms == NULL)
	{
		printf(" Memory is not allocated ");
		exit(1);
	}
	for(j=0; j<=num_atoms; j++)
	{
		atoms[j]= malloc(60 * sizeof(char));
		if(atoms[j] == NULL)
		{
			printf(" the memory is not allocated ");
			exit(1);
		}
	}
	
	fp_N=fopen("Natoms.txt", "r");
	array_Index=0;
	j=1;
	scan(fp_N, array_Index, j, atoms, num_atoms);
	fclose(fp_N);
		
	fp_CA=fopen("CAatoms.txt", "r");
	array_Index=1;
	j=2;
	scan(fp_CA, array_Index, j, atoms, num_atoms);
	fclose(fp_CA);
	
	fp_C=fopen("Catoms.txt", "r");
	array_Index=2;
	j=3;
	scan(fp_C, array_Index, j, atoms, num_atoms);
	fclose(fp_C);
	
	fp_CB=fopen("CBatoms.txt", "r");
	array_Index=3;
	j=4;
	struct_ptr=scan(fp_CB, array_Index, j, atoms, num_atoms);
		
	fp_Result=fopen("Chirality_Result.txt", "w");		 /*the final result is written to the file*/
	
	num_atoms=num_atoms/4;
	
	for(j=0;j<num_atoms;j++)
	{
		if(struct_ptr->theta <0 && struct_ptr->theta != 0)
		{
		
		fprintf(fp_Result, "%s\t%d\t %f\t D\n", struct_ptr->Aminoacid, struct_ptr->aa_Num, struct_ptr->theta);
		printf("\e[1;31m%s\t%d\t %f\t D\033[0m\n", struct_ptr->Aminoacid, struct_ptr->aa_Num, struct_ptr->theta);

		}
		else if(struct_ptr->theta >0 && struct_ptr->theta !=0)
		{
			fprintf(fp_Result, "%s\t%d\t%f\t L \n", struct_ptr->Aminoacid, struct_ptr->aa_Num, struct_ptr->theta);
			printf( "\033[0m%s\t%d\t%f\t L \n", struct_ptr->Aminoacid, struct_ptr->aa_Num, struct_ptr->theta);
		}
		struct_ptr++;
			
	}
	fclose(fp_Result);

	remove("file_Atoms.txt");
	remove("file_AAcoordinates.txt");
	remove("Natoms.txt");
	remove("CAatoms.txt");
	remove("Catoms.txt");
	remove("CBatoms.txt");
	
	return 0;
		
	
}
 
 
 /*extracting the coordinates of atoms into an array*/
struct Chirality *scan(FILE *fp , int i, int file_num, char **atoms, int num_atoms)
	{
		
		int j;
		static struct Chirality *ptr=NULL;
		
		while(feof(fp) == 0 && i<num_atoms)
		{	fgets(atoms[i], 60, fp);
			i=i+4;
		}
		if (i<(num_atoms+4) && file_num == 4)
		{
			ptr=dihedral(atoms,i);
			return ptr;
		}
	
		return ptr;

	}
 

/*this functions calculate the dihedral angle and determines the chirality of CA atom*/

struct Chirality * dihedral(char **atoms, int num) 
{
	int aminoacid_Num[num], aa_Num[num], i=0, k=0, j=0, atom_num;
	struct Chirality *result;
	struct Chirality *ptr;
	char atom_type[num][5], AA[num][5],aminoacid[num][5];
	float x[num],y[num], z[num];
	float A, B, C;
	float a[num][3], b[num][3], c[num][3];
	float normal1[3], normal2[3],s1Ds2, s1sqrt, s2sqrt, theta, s1Cs2[3],s1s2Db;
	FILE *fp_Result;	
		
	result=(struct Chirality *)malloc( num * sizeof( struct Chirality ));
	
	if(result == NULL)
	{
		printf(" the memory has not been allocated");
		exit(1);
	}
	
	for(i=0;i<(num-3);i++)
	{
		sscanf(atoms[i], "%s %s %d %f %f %f", atom_type[i], AA[i], &aa_Num[i], &x[i], &y[i], &z[i]);
			                                                                                         
	}
	for(i=0;i<num;i=i+4)
	{
	  
		strncpy(aminoacid[j], AA[i], 5);
		aminoacid_Num[j]=aa_Num[i];
		j++;
	}		
     	j=0;
		
        for(i=0;i<num;i++)
        {

        	a[j][0] = (x[i+1]-x[i]);
        	a[j][1] = (y[i+1]-y[i]);
        	a[j][2] = (z[i+1]-z[i]);
		
                i=i+1;

                b[j][0] = (x[i+2]-x[i]);
                b[j][1] = (y[i+2]-y[i]);
                b[j][2] = (z[i+2]-z[i]);
	
                c[j][0] = (x[i+1]-x[i]);
                c[j][1] = (y[i+1]-y[i]);
                c[j][2] = (z[i+1]-z[i]);
	
	
                i=i+2;
		j++;

        }
	atom_num=num/4;

       for(j=0; j<atom_num; j++  ) /*calculating the normal between N-CA-CB and C-CA-CB planes*/
        {
		A=sqrt((pow(a[j][0],2) + pow(a[j][1],2)	+ pow(a[j][2],2)));
		B=sqrt((pow(b[j][0],2) + pow(b[j][1],2)+ pow(b[j][2],2)));
		C=sqrt((pow(c[j][0],2) + pow(c[j][1],2)+ pow(c[j][2],2)));
		

		if(A>1.4 && B >1.5 && C > 1.5)
		{
			normal1[0]= (b[j][2] * a[j][1]) - (b[j][1] * a[j][2]);
              	  	normal1[1]= -1*((a[j][0] * b[j][2]) - (b[j][0] * a[j][2]));
              		normal1[2]= (a[j][0] * b[j][1]) - (b[j][0] * a[j][1]);

              	 	normal2[0]= (c[j][2] * b[j][1]) - (c[j][1] * b[j][2]);
              	  	normal2[1]= -1*((c[j][2] * b[j][0]) - (c[j][0] * b[j][2]));
                	normal2[2]= (c[j][1] * b[j][0]) - (c[j][0] * b[j][1]);
	
             		s1Ds2 = (normal1[0] * normal2[0]) + (normal1[1] * normal2[1]) + (normal1[2] * normal2[2]);
		    	s1sqrt=sqrt(pow(normal1[0], 2) + pow(normal1[1], 2) + pow(normal1[2], 2));
              	 	s2sqrt=sqrt(pow(normal2[0], 2) + pow(normal2[1], 2) + pow(normal2[2], 2));
              	
			theta=(acos((s1Ds2/(s1sqrt * s2sqrt ))))*57.3;          
        	
			s1Cs2[0]=((normal2[2]*normal1[1])-(normal1[2]*normal2[1]));
			s1Cs2[1]=-1*((normal2[2]*normal1[0])-(normal2[0]*normal1[2]));
			s1Cs2[2]=((normal2[1]*normal1[0])-(normal2[0]*normal1[1]));
	
			s1s2Db=(s1Cs2[0]*b[j][0])+(s1Cs2[1]*b[j][1])+(s1Cs2[2]*b[j][2]);

			if(s1s2Db<0)
			{
				theta= (-1*theta);
				result[k].theta=theta;
								
			}
			else
			{
				result[k].theta=theta;
			}

			strcpy(result[k].Aminoacid, aminoacid[j]);
                        result[k].aa_Num=aminoacid_Num[j];

			k++;
		}
		
	}
			
	ptr=result;
	return ptr;
}

 /*this function scans for the aminoacid coordinates from the pdb file and stores it into another temporary file */

int file(int num_atoms)                                                                       
{

	int file_Aminoacid_num[num_atoms], pre_AA_num[10], count=1,row;
	int i=0, j=0, k=0;
	char line[60];
	char file_Atom[3], file_AA[num_atoms][6], file_String[num_atoms][40];
	char pre_AA[10][6];
	float file_X, file_Y, file_Z;
	FILE *fp_AAcoordinates, *fp_Atom;
	
	fp_AAcoordinates=fopen("file_AAcoordinates.txt", "w");
	fp_Atom=fopen("file_Atoms.txt", "r");
	
	while( fgets(line, 40, fp_Atom) != NULL && feof(fp_Atom) ==0)
	{
		sscanf(line, "%s %s %d %f %f %f", file_Atom, file_AA[i], &file_Aminoacid_num[i], &file_X, &file_Y, &file_Z);
				
		if((j!=0)&&(strcmp(pre_AA[j-1],file_AA[i])==0)&&(pre_AA_num[j-1]==file_Aminoacid_num[i]))
		{
			if(count==3)
			{
				if(pre_AA_num[0]==file_Aminoacid_num[i])
				{
					count=count+1;
				}
			}
			else
				count=count+1;
		}
		else if((j!=0)&&(pre_AA_num[j-1]!=file_Aminoacid_num[i]))
		{
			count=1;
			for(k=0;k<6;k++)
			{
				strcpy(pre_AA[k], "\0");
			}
			j=0;
		  }
		
		strncpy(pre_AA[j],file_AA[i], 6);
		pre_AA_num[j]=file_Aminoacid_num[i];
		strncpy(file_String[i],line, 40);

		if(count==4)
		{
			for(row=i-3;row<=i;row++)
			{
				fputs( file_String[row], fp_AAcoordinates);
			}
			count=1;
			j=0;
		}
		i++; 
		j++;                          
	}
	fclose(fp_Atom);
	fclose(fp_AAcoordinates);
	return i;
}
